# frozen_string_literal: true

module  Apiphobic
module  Parameters
class   Include
  attr_accessor :raw_parameters

  def initialize(raw_parameters)
    self.raw_parameters = raw_parameters ? raw_parameters.split(',') : []
  end

  def present?
    raw_parameters.any?
  end

  def to_a
    @to_a ||= map_inclusions(
                raw_parameters
                  .reject(&:empty?)
                  .compact,
              )
  end

  private

  # rubocop:disable Metrics/PerceivedComplexity,
  # rubocop:disable Metrics/BlockNesting
  def map_inclusions(array)
    mapped = array.each_with_object([{}]) do |inclusion, memo|
      if inclusion.is_a?(::Hash)
        memo[0] = inclusion
      else
        first, _match, rest = inclusion.partition('.')

        if rest == ''
          memo << first
        else
          memo.delete(first)

          memo[0][first] = if memo[0][first].nil?
                             rest
                           else
                             map_inclusions(Array(memo[0][first]) << rest)
                           end
        end
      end
    end

    mapped.shift if mapped[0] == {}

    mapped
  end
  # rubocop:enable Metrics/BlockNesting
  # rubocop:enable Metrics/PerceivedComplexity,
end
end
end
