# frozen_string_literal: true

require 'forwardable'

require 'apiphobic/resource/processors/filtering'
require 'apiphobic/resource/processors/including'
require 'apiphobic/resource/processors/indexing'
require 'apiphobic/resource/processors/paging'
require 'apiphobic/resource/processors/sorting'

module  Apiphobic
module  Resource
class   Model
  extend Forwardable

  DEFAULT_PROCESSORS = %w{including}.freeze

  attr_accessor :id,
                :resource,
                :parameters
  attr_reader   :processors

  def_delegators :processed,
                 :assign_attributes,
                 :delete,
                 :save!,
                 :save,
                 :subscribe

  def initialize(resource:, id:, parameters:, **options)
    self.id         = id
    self.resource   = resource
    self.parameters = parameters.dup
    self.processors = options.fetch(:processors, self.class::DEFAULT_PROCESSORS)
  end

  def processed
    @processed ||= begin
      processed  = processors.inject(resource) do |processed_resource, processor|
        processor.processed(processed_resource, parameters)
      end

      return processed if instance_of?(::Apiphobic::Resource::Collection)

      processed.find_by(id: id) || processed.new
    end
  end

  def meta
    @meta ||= processors.inject({}) do |metadata, processor|
      metadata.merge processor.meta(processed, parameters)
    end
  end

  def processors=(other)
    @processors = other.map do |processor|
      Object.const_get("::Apiphobic::Resource::Processors::#{processor.capitalize}")
    end
  end
end
end
end
